package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.model.Material
import dev.fyloz.colorrecipesexplorer.model.MixMaterial
import dev.fyloz.colorrecipesexplorer.model.MixMaterialDto
import dev.fyloz.colorrecipesexplorer.model.mixMaterial
import dev.fyloz.colorrecipesexplorer.repository.MixMaterialRepository
import org.springframework.context.annotation.Lazy
import org.springframework.stereotype.Service

interface MixMaterialService : ModelService<MixMaterial, MixMaterialRepository> {
    /** Checks if one or more mix materials have the given [material]. */
    fun existsByMaterial(material: Material): Boolean

    /** Creates [MixMaterial]s from the givens [MixMaterialDto]. */
    fun create(mixMaterials: Set<MixMaterialDto>): Set<MixMaterial>

    /** Creates a [MixMaterial] from a given [MixMaterialDto]. */
    fun create(mixMaterial: MixMaterialDto): MixMaterial

    /** Updates the [quantity] of the given [mixMaterial]. */
    fun updateQuantity(mixMaterial: MixMaterial, quantity: Float): MixMaterial
}

@Service
class MixMaterialServiceImpl(
    mixMaterialRepository: MixMaterialRepository,
    @Lazy val materialService: MaterialService
) : AbstractModelService<MixMaterial, MixMaterialRepository>(mixMaterialRepository), MixMaterialService {
    override fun existsByMaterial(material: Material): Boolean = repository.existsByMaterial(material)
    override fun create(mixMaterials: Set<MixMaterialDto>): Set<MixMaterial> =
        mixMaterials.map(::create).toSet()

    override fun create(mixMaterial: MixMaterialDto): MixMaterial =
        mixMaterial(
            material = materialService.getById(mixMaterial.materialId),
            quantity = mixMaterial.quantity,
            position = mixMaterial.position
        )

    override fun updateQuantity(mixMaterial: MixMaterial, quantity: Float) =
        update(mixMaterial.apply {
            this.quantity = quantity
        })
}
