package dev.fyloz.colorrecipesexplorer.service.utils

/** Returns a list containing the result of the given [transform] applied to each item of the [Iterable]. If the given [transform] throws, the [Throwable] will be passed to the given [throwableConsumer]. */
inline fun <T, R, reified E : Throwable> Iterable<T>.mapMayThrow(
    throwableConsumer: (E) -> Unit = {},
    transform: (T) -> R
): List<R> = this.mapNotNull {
    try {
        transform(it)
    } catch (th: Throwable) {
        if (th is E) {
            throwableConsumer(th)
            null
        } else {
            throw th
        }
    }
}

/** Clears and fills the given [MutableCollection] with the given [elements]. */
fun <T> MutableCollection<T>.setAll(elements: Collection<T>) {
    this.clear()
    this.addAll(elements)
}
